package yuehaoxitong

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/yuehaoxitong/model"
)

type schedule struct {
	Table
	queue     queue
	userQueue userQueue
	reception reception
	volume    volume
	portal    string `data:"true"`
	plan      string `name:"plan begin_time@date end_time@date"`
}

func (s schedule) Create(m *ice.Message, arg ...string) {
	s.Transaction(m, func() {
		msg := m.Cmd(s.volume, s.SelectForUpdate, model.UID, m.Option(model.VOLUME_UID))
		if m.Warn(kit.Int(msg.Append(model.COUNT)) >= kit.Int(msg.Append(model.AMOUNT)), "本服务已约满") {
			return
		}
		m.Cmd(s.volume, s.AddCount, m.Option(model.VOLUME_UID), "1")
		s.Table.Create(m, arg...)
	})
	if !m.IsErr() {
		m.Option(model.UID, m.Result())
		s.SendTemplate(m, "约号成功")
	}
}
func (s schedule) Call(m *ice.Message, arg ...string) {
	s.Orders(m, model.TAKE_TIME).Limit(m, 1)
	s.SelectByStatus(m, m.Option(model.QUEUE_UID), kit.Format(ScheduleTake))
	// s.Select(m, kit.Simple(m.OptionSimple(model.QUEUE_UID, model.RECEPTION_UID), mdb.STATUS, ScheduleTake)...)
	if m.WarnNotFound(m.Length() == 0) {
		return
	}
	s.ChangeStatus(m, m.Append(model.UID), "", ScheduleTake, ScheduleCall, arg...)
	s.SendTemplate(m, "服务已叫号")
}
func (s schedule) SelectByStatus(m *ice.Message, arg ...string) {
	s.Tables(m, s.volume).FieldsWithCreatedAT(m, s,
		model.SCHEDULE_STATUS, s.Key(s.volume, model.BEGIN_TIME), s.Key(s.volume, model.END_TIME), model.VOLUME_UID, s.Key(s, model.RECEPTION_UID), model.USER_UID,
		s.Key(s, model.QUEUE_UID),
	).Orders(m, s.Desc(model.BEGIN_TIME))
	s.Select(m, kit.Simple(s.Key(s, model.QUEUE_UID), arg[0], model.STATUS, arg[1], arg[2:])...)
	s.RewriteAppend(m)
	s.SelectJoin(m, s.reception)
	s.SelectJoinUser(m)
	m.PushAction().Action()
}
func (s schedule) List(m *ice.Message, arg ...string) {
	if s.CheckRole(m.Options(model.QUEUE_UID, arg[0]), UserQueueCreator, UserQueueServer) {
		if len(arg) == 1 {
			s.Select(m, model.QUEUE_UID, arg[0])
		} else if len(arg) == 2 {
			s.SelectDetail(m, model.QUEUE_UID, arg[0], model.UID, arg[1])
		}
	} else {
		if len(arg) == 1 {
			s.Select(m, model.QUEUE_UID, arg[0], model.USER_UID, m.Option(model.USER_UID))
		} else if len(arg) == 2 {
			s.SelectDetail(m, model.QUEUE_UID, arg[0], model.USER_UID, m.Option(model.USER_UID), model.UID, arg[1])
		}
	}
	m.RenameAppend(model.STATUS, model.SCHEDULE_STATUS)
	s.RewriteAppend(m)
	s.SelectJoin(m, s.reception)
	s.SelectJoinUser(m)
	m.PushAction().Action()
}
func (s schedule) Cancel(m *ice.Message, arg ...string) {
	s.Transaction(m, func() {
		m.Cmd(s.volume, s.AddCount, m.Option(model.VOLUME_UID), "-1")
		s.ChangeStatus(m, m.Option(model.UID), "", SchedulePlan, ScheduleTake, arg...)
	})
	if !m.IsErr() {
		s.SendTemplate(m, "约号取消")
	}
}
func (s schedule) Take(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, m.Option(model.UID), "", SchedulePlan, ScheduleTake, arg...)
	s.SendTemplate(m, "取号取消")
}
func (s schedule) Expire(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, m.Option(model.UID), "", ScheduleCall, ScheduleExpire, arg...)
	s.SendTemplate(m, "服务已过号")
}
func (s schedule) Finish(m *ice.Message, arg ...string) {
	s.ChangeStatus(m, m.Option(model.UID), "", ScheduleCall, ScheduleFinish, arg...)
	s.SendTemplate(m, "服务已完成")
}

func init() { ice.TeamCtxCmd(schedule{Table: newTable{}}) }

func (s schedule) ChangeStatus(m *ice.Message, uid, key string, from, to ScheduleStatus, arg ...string) *ice.Message {
	kit.If(key == "", func() { key = m.ActionKey() + "_time" })
	s.Table.ChangeStatus(m, uid, int(from), int(to), kit.TransArgValueTime(append(arg, key, m.Time()), key)...)
	return m
}
func (s schedule) SendTemplate(m *ice.Message, arg ...string) {
	queue := m.Cmd(s.queue, s.Select, model.UID, m.Option(model.QUEUE_UID))
	reception := m.Cmd(s.reception, s.Select, model.UID, m.Option(model.RECEPTION_UID))
	m.Option(model.QUEUE_NAME, queue.Append(model.NAME)+" "+reception.Append(model.NAME))
	s.Table.SendTemplate(m, "", m.Option(model.USER_UID), arg[0])
}

type ScheduleStatus int

const (
	SchedulePlan ScheduleStatus = iota
	ScheduleCancel
	ScheduleTake
	ScheduleCall
	ScheduleExpire
	ScheduleFinish
)

var ScheduleStatusList = map[ScheduleStatus]string{
	SchedulePlan:   "plan",
	ScheduleCancel: "cancel",
	ScheduleTake:   "take",
	ScheduleCall:   "call",
	ScheduleExpire: "expire",
	ScheduleFinish: "finish",
}

func (s ScheduleStatus) String() string { return ScheduleStatusList[s] }
